/********************************************************************************
*                                                                               *
*                      F X   M O V I E   M A N A G E R                          *
*                                                                               *
*********************************************************************************
* Copyright(C) 2008,2012 by Luigi Ziviani <ziviani_luigi@yahoo.it>.             *
* All Rights Reserved.                                                          *
*********************************************************************************
* Id: fxarchive.cpp,   v 1.4                       2008/01/01                   *
*********************************************************************************
* This file is part of FXMOVIEMANAGER                                           *
* This program is free software; you can redistribute it and/or modify          *
* it under the terms of the GNU General Public License as published by          *
* the Free Software Foundation; either version 3 of the License, or             *
* (at your option) any later version.                                           *
*                                                                               *
* This program is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                 *
* GNU General Public License for more details.                                  *
* You should have received a copy of the GNU General Public License             *
* along with this program; if not, write to the Free Software                   *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA     *
********************************************************************************/
#include <fx.h>
#include <signal.h>

#include "icons.h"
#include "fxarchive.h"
#include "config.h"

// Message Map for the Archive Window class
FXDEFMAP(ArchiveDialog) ArchiveDialogMap[]={

  //________Message_Type_______________________ID______________________Message_Handler_______
  FXMAPFUNC(SEL_CLIPBOARD_LOST,				0,			ArchiveDialog::onClipboardLost),
  FXMAPFUNC(SEL_CLIPBOARD_GAINED,			0,			ArchiveDialog::onClipboardGained),
  FXMAPFUNC(SEL_CLIPBOARD_REQUEST,			0,			ArchiveDialog::onClipboardRequest),
  FXMAPFUNC(SEL_COMMAND,		   ArchiveDialog::ID_CLIPBOARD_COPY,	ArchiveDialog::onCmdCopyCut),
  FXMAPFUNC(SEL_COMMAND,		   ArchiveDialog::ID_CLIPBOARD_CUT,	ArchiveDialog::onCmdCopyCut),
  FXMAPFUNC(SEL_COMMAND,		   ArchiveDialog::ID_CLIPBOARD_PASTE,	ArchiveDialog::onCmdPaste),
  FXMAPFUNC(SEL_UPDATE,			   ArchiveDialog::ID_CLIPBOARD_PASTE,	ArchiveDialog::onUpdPaste),

  FXMAPFUNC(SEL_COMMAND,		   ArchiveDialog::ID_QUIT,  		ArchiveDialog::onCmdQuit),
  FXMAPFUNC(SEL_UPDATE,			   ArchiveDialog::ID_MAINWINDOW,    	ArchiveDialog::onUpdTitle),

  FXMAPFUNC(SEL_COMMAND,		   ArchiveDialog::ID_COMPRESS_TAR, 	 ArchiveDialog::onCmdCompressTAR),
  FXMAPFUNC(SEL_UPDATE,			   ArchiveDialog::ID_COMPRESS_TAR,	ArchiveDialog::onUpdSelectedTar),
  FXMAPFUNC(SEL_COMMAND,		   ArchiveDialog::ID_COMPRESS_GZ,  	ArchiveDialog::onCmdCompressGZ),
  FXMAPFUNC(SEL_UPDATE,			   ArchiveDialog::ID_COMPRESS_GZ,	ArchiveDialog::onUpdSelectedCompress),
  FXMAPFUNC(SEL_COMMAND,		   ArchiveDialog::ID_COMPRESS_BZ2,  	ArchiveDialog::onCmdCompressBZ2),
  FXMAPFUNC(SEL_UPDATE,			   ArchiveDialog::ID_COMPRESS_BZ2,	ArchiveDialog::onUpdSelectedCompress),
  FXMAPFUNC(SEL_COMMAND,		   ArchiveDialog::ID_COMPRESS_XZ, 	ArchiveDialog::onCmdCompressXZ),
  FXMAPFUNC(SEL_UPDATE,			   ArchiveDialog::ID_COMPRESS_XZ,	ArchiveDialog::onUpdSelectedCompress),
  FXMAPFUNC(SEL_COMMAND,		   ArchiveDialog::ID_COMPRESS_ZIP,  	ArchiveDialog::onCmdCompressZIP),
  FXMAPFUNC(SEL_UPDATE,			   ArchiveDialog::ID_COMPRESS_ZIP,	ArchiveDialog::onUpdSelectedCompress),
  FXMAPFUNC(SEL_COMMAND,		   ArchiveDialog::ID_COMPRESS_7ZIP,	ArchiveDialog::onCmdCompress7ZIP),
  FXMAPFUNC(SEL_UPDATE,			   ArchiveDialog::ID_COMPRESS_7ZIP,	ArchiveDialog::onUpdSelectedCompress),

  FXMAPFUNC(SEL_COMMAND,		   ArchiveDialog::ID_EXTRACT,   	ArchiveDialog::onCmdExtract),
  FXMAPFUNC(SEL_UPDATE,			   ArchiveDialog::ID_EXTRACT,		ArchiveDialog::onUpdSelectedExtract),

  FXMAPFUNC(SEL_COMMAND,		   ArchiveDialog::ID_FORCEEXTRACT,  	ArchiveDialog::onCmdForceExtract),
  FXMAPFUNC(SEL_UPDATE,			   ArchiveDialog::ID_FORCEEXTRACT,	ArchiveDialog::onUpdSelectedExtract),

  FXMAPFUNC(SEL_COMMAND,		   ArchiveDialog::ID_COMMAND,   	ArchiveDialog::onCmdCommand),
  FXMAPFUNC(SEL_COMMAND,		   ArchiveDialog::ID_TERMINAL,   	ArchiveDialog::onCmdTerminal),

  FXMAPFUNC(SEL_COMMAND,		   ArchiveDialog::ID_OPEN,		ArchiveDialog::onCmdOpen),
  FXMAPFUNC(SEL_COMMAND,		   ArchiveDialog::ID_OPENWITH,		ArchiveDialog::onCmdOpenWith),

  FXMAPFUNC(SEL_DOUBLECLICKED, 	   	   ArchiveDialog::ID_FILELIST,		ArchiveDialog::onCmdFilelist),
  FXMAPFUNC(SEL_RIGHTBUTTONRELEASE,	   ArchiveDialog::ID_FILELIST,		ArchiveDialog::onFileListPopup),
  FXMAPFUNC(SEL_COMMAND,		   ArchiveDialog::ID_DIRLIST,		ArchiveDialog::onCmdDirlist),

  FXMAPFUNC(SEL_COMMAND,		   ArchiveDialog::ID_FILERENAME,	ArchiveDialog::onCmdFileRename),
  FXMAPFUNC(SEL_UPDATE,			   ArchiveDialog::ID_FILERENAME,	ArchiveDialog::onUpdFileSelected),

  FXMAPFUNC(SEL_COMMAND,		   ArchiveDialog::ID_FILEDELETE,	ArchiveDialog::onCmdFileDelete),
  FXMAPFUNC(SEL_UPDATE,			   ArchiveDialog::ID_FILEDELETE,	ArchiveDialog::onUpdFileSelected),

  FXMAPFUNC(SEL_COMMAND,		   ArchiveDialog::ID_ABOUT,   	    	ArchiveDialog::onCmdAbout),
  };


// Macro for the ArchiveApp class hierarchy implementation
FXIMPLEMENT(ArchiveDialog,FXMainWindow,ArchiveDialogMap,ARRAYNUMBER(ArchiveDialogMap))

// Create Archive dialog
ArchiveDialog::ArchiveDialog(FXApp *a)
	:FXMainWindow(a,"FXArchive Utility",NULL,NULL,DECOR_ALL,0,0,700,400){

  // set icon on the main window
  logoicon=new FXPNGIcon(getApp(),fxar64);
  logominiicon=new FXPNGIcon(getApp(),fxar16);
  setIcon(logoicon);
  setMiniIcon(logominiicon);

  setTarget(this);
  setSelector(ID_MAINWINDOW);

  accepticon=new FXGIFIcon(getApp(),m_accept);
  cancelicon=new FXGIFIcon(getApp(),m_cancel);
  quiticonb=new FXGIFIcon(getApp(),m_quit_big);
  extracticon=new FXGIFIcon(getApp(),m_archivextr);
  extracticonb=new FXGIFIcon(getApp(),m_archivextr_big);
  commandicon=new FXGIFIcon(getApp(),m_archiverun);
  commandiconb=new FXGIFIcon(getApp(),m_archiverun_big);
  compressicons1=new FXGIFIcon(getApp(),mime_archive1_small);
  compressicons2=new FXGIFIcon(getApp(),mime_archive2_small);
  compressicons3=new FXGIFIcon(getApp(),mime_archive3_small);
  compressicons4=new FXGIFIcon(getApp(),mime_archive4_small);
  compressicons5=new FXGIFIcon(getApp(),mime_archive5_small);
  compressicon1=new FXGIFIcon(getApp(),mime_archive1);
  compressicon2=new FXGIFIcon(getApp(),mime_archive2);
  compressicon3=new FXGIFIcon(getApp(),mime_archive3);
  compressicon4=new FXGIFIcon(getApp(),mime_archive4);
  compressicon5=new FXGIFIcon(getApp(),mime_archive5);
  m_refreshicon=new FXGIFIcon(getApp(),m_refresh);
  m_terminalicon=new FXGIFIcon(getApp(),m_terminal);
  m_appendicon=new FXGIFIcon(getApp(),m_append);
  m_cuticon=new FXGIFIcon(getApp(),m_cut);  
  m_copyicon=new FXGIFIcon(getApp(),m_copy);
  m_pasteicon=new FXGIFIcon(getApp(),m_paste);
  m_deleteicon=new FXGIFIcon(getApp(),m_delete);
  m_renamediricon=new FXGIFIcon(getApp(),m_renamedir);
  m_consoleicon=new FXGIFIcon(getApp(),m_console);
  // filelist header
  m_usricon=new FXGIFIcon(getApp(),m_usr);
  m_grpicon=new FXGIFIcon(getApp(),m_grp);
  m_prmicon=new FXGIFIcon(getApp(),m_prm);
  m_dimicon=new FXGIFIcon(getApp(),m_dim);
  m_daticon=new FXGIFIcon(getApp(),m_dat);
  m_diricon=new FXGIFIcon(getApp(),m_dir);

  if(!readConfig()){
	  printf("Configuration not loaded!");
  }
	// menubar
  menubar = new FXMenuBar(this,LAYOUT_SIDE_TOP|LAYOUT_FILL_X);

  dialogframe=new FXVerticalFrame(this,LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,0, 0,0,0,0,0,0);
  filesframe=new FXHorizontalFrame(dialogframe,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,0, 0,0,0,0,0,0);
  splitter=new FXSplitter(filesframe,LAYOUT_SIDE_TOP|FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_FILL_Y|SPLITTER_TRACKING|SPLITTER_HORIZONTAL|SPLITTER_REVERSED);
  dirlist=new FXDirList(splitter,this,ID_DIRLIST,LAYOUT_FILL_X|LAYOUT_FILL_Y|TREELIST_SHOWS_LINES|TREELIST_SHOWS_BOXES|TREELIST_BROWSESELECT|DIRLIST_NO_OWN_ASSOC,0,0,0,0);
  filelist=new fxmmFileList(splitter,this,ID_FILELIST,LAYOUT_FILL_X|LAYOUT_FILL_Y|ICONLIST_MINI_ICONS|ICONLIST_AUTOSIZE|FILELIST_NO_OWN_ASSOC,0,0,500,0);


  // Arrange menu
  menu_arrange=new FXMenuPane(this);
  title_arrange=new FXMenuTitle(menubar,"&Arrange",NULL,menu_arrange);
  radio_det=new FXMenuRadio(menu_arrange,"Detailed &List\t\tView as detailed list.",filelist,FXFileList::ID_SHOW_DETAILS);
  radio_sma=new FXMenuRadio(menu_arrange,"&Small icons\t\tView as small icons.",filelist,FXFileList::ID_SHOW_MINI_ICONS);
  radio_big=new FXMenuRadio(menu_arrange,"&Big icons\t\tView as big icons.",filelist,FXFileList::ID_SHOW_BIG_ICONS);

  // Show/Hide menu
  menu_show=new FXMenuPane(this);
  title_show=new FXMenuTitle(menubar,"&Show/Hide",NULL,menu_show);
  check_fil=new FXMenuCheck(menu_show,"Hidden &Directories\t\tShow hidden directories.",dirlist,FXDirList::ID_TOGGLE_HIDDEN);
  check_dir=new FXMenuCheck(menu_show,"Hidden &Files\t\tShow hidden files and directories.",filelist,FXFileList::ID_TOGGLE_HIDDEN);
  new FXMenuSeparator(menu_show);
  check_img=new FXMenuCheck(menu_show,"&Preview Images\t\tShow thumbnail images.",filelist,FXFileList::ID_TOGGLE_IMAGES);

  // About menu
  menu_about=new FXMenuPane(this);
  title_about=new FXMenuTitle(menubar,"&About",NULL,menu_about,LAYOUT_RIGHT);
  cmd_about=new FXMenuCommand(menu_about,"&About\t\tAbout FXArchive",NULL,this,ID_ABOUT);

  // buttons
  buttonsframe=new FXVerticalFrame(splitter,LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_Y|PACK_UNIFORM_WIDTH,0,0,0,0, 0,0,0,0);
#ifndef DISABLE_TAR
  button_ta=new FXButton(buttonsframe,tr(" &TAR"),compressicon2,this,ID_COMPRESS_TAR,BUTTON_DEFAULT|BUTTON_NORMAL|LAYOUT_CENTER_X|LAYOUT_CENTER_Y,0,0,0,0,6,6);
#endif
#ifndef DISABLE_GZIP
  button_gz=new FXButton(buttonsframe,tr(" &GZ"),compressicon3,this,ID_COMPRESS_GZ,BUTTON_DEFAULT|BUTTON_NORMAL|LAYOUT_CENTER_X|LAYOUT_CENTER_Y,0,0,0,0,6,6);
#endif
#ifndef DISABLE_BZIP2
  button_bz=new FXButton(buttonsframe,tr(" &BZ2"),compressicon4,this,ID_COMPRESS_BZ2,BUTTON_DEFAULT|BUTTON_NORMAL|LAYOUT_CENTER_X|LAYOUT_CENTER_Y,0,0,0,0,6,6);
#endif
#ifndef DISABLE_XZIP
  button_xz=new FXButton(buttonsframe,tr(" &XZ"),compressicon1,this,ID_COMPRESS_XZ,BUTTON_DEFAULT|BUTTON_NORMAL|LAYOUT_CENTER_X|LAYOUT_CENTER_Y,0,0,0,0,6,6);
#endif
#ifndef DISABLE_ZIP
  button_zi=new FXButton(buttonsframe,tr(" &ZIP"),compressicon5,this,ID_COMPRESS_ZIP,BUTTON_DEFAULT|BUTTON_NORMAL|LAYOUT_CENTER_X|LAYOUT_CENTER_Y,0,0,0,0,6,6);
#endif
#ifndef DISABLE_SEVENZIP
  button_7z=new FXButton(buttonsframe,tr(" &7ZIP"),compressicon5,this,ID_COMPRESS_7ZIP,BUTTON_DEFAULT|BUTTON_NORMAL|LAYOUT_CENTER_X|LAYOUT_CENTER_Y,0,0,0,0,6,6);
#endif
  button_ex=new FXButton(buttonsframe,tr(" &Ext"),extracticonb,this,ID_EXTRACT,BUTTON_DEFAULT|BUTTON_NORMAL|LAYOUT_CENTER_X|LAYOUT_CENTER_Y,0,0,0,0,6,6);

  button_cm=new FXButton(buttonsframe,tr(" &Cmd"),commandiconb,this,ID_FORCEEXTRACT,BUTTON_DEFAULT|BUTTON_NORMAL|LAYOUT_CENTER_X|LAYOUT_CENTER_Y,0,0,0,0,6,6);

  button_qu=new FXButton(buttonsframe,tr(" &Quit"),quiticonb,this,ID_QUIT,BUTTON_DEFAULT|BUTTON_NORMAL|LAYOUT_CENTER_X|LAYOUT_CENTER_Y,0,0,0,0,6,6);

  new FXHorizontalSeparator(dialogframe,SEPARATOR_GROOVE|LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X);

  FXHorizontalFrame *copyright=new FXHorizontalFrame(dialogframe,LAYOUT_BOTTOM|LAYOUT_FILL_X,0,0,0,0,5,5,5,5);
  FXLabel *label = new FXLabel(copyright,"FXArchive - Compression/Decompression Utility",NULL,LAYOUT_FILL_X|LAYOUT_CENTER_Y|JUSTIFY_LEFT,0,0,0,0,15);
  // set the same look of separators
  label->disable();
}

// Destroy
ArchiveDialog::~ArchiveDialog(){
  delete accepticon;
  delete cancelicon;
  delete quiticonb;
  delete extracticon;
  delete extracticonb;
  delete commandicon;
  delete commandiconb;
  delete compressicon1;
  delete compressicon2;
  delete compressicon3;
  delete compressicon4;
  delete compressicon5;
  delete compressicons1;
  delete compressicons2;
  delete compressicons3;
  delete compressicons4;
  delete compressicons5;
  delete m_refreshicon;
  delete m_terminalicon;
  delete m_appendicon;
  delete m_cuticon;
  delete m_copyicon;
  delete m_pasteicon;
  delete m_deleteicon;
  delete m_consoleicon;
  delete m_renamediricon;
  // filelist header
  delete m_usricon;
  delete m_grpicon;
  delete m_prmicon;
  delete m_dimicon;
  delete m_daticon;
  delete m_diricon;
  }


// Execute Compress Command
FXbool ArchiveDialog::executeCompressCommand(fxmmFileList* panel, const FXString& dlg_cmd, const FXString& dlg_ext, const FXString& dlg_label, FXint dlg_form){

    FXString filetocomp,file, executable;
    for(FXint i=0; i<panel->getNumItems(); i++){
      if(panel->isItemSelected(i)){
        file=panel->getItemFilename(i);
        if(file=="..") continue;
        filetocomp=FXPath::name(file);

		if (dlg_form==1)    { executable="cd "+FXPath::enquote(panel->getDirectory())+"; "+dlg_cmd+filetocomp+dlg_ext+filetocomp+" &"; }
		else if(dlg_form==2){ executable="cd "+FXPath::enquote(panel->getDirectory())+"; "+dlg_cmd+filetocomp+" &"; }

		FXDialogBox dialog(this,dlg_label,DECOR_TITLE|DECOR_BORDER);
		FXVerticalFrame* content=new FXVerticalFrame(&dialog,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,10,10,10,10,10,10);
		new FXLabel(content,filetocomp,NULL,LAYOUT_FILL_X);
  	  	FXHorizontalFrame* buttons=new FXHorizontalFrame(content,LAYOUT_FILL_X|PACK_UNIFORM_WIDTH,0,0,0,0,0,0,0,0);
  	  	new FXButton(buttons,"&YES",accepticon,&dialog,FXDialogBox::ID_ACCEPT,BUTTON_INITIAL|ICON_BEFORE_TEXT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT);
  	  	new FXButton(buttons,"&Cancel",cancelicon,&dialog,FXDialogBox::ID_CANCEL,ICON_BEFORE_TEXT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT,0,0,0,0,20,20);
  	  	dialog.create();

  	  	if(dialog.execute()){
	        int ret = system(executable.text());
	        if (WIFSIGNALED(ret) && (WTERMSIG(ret) == SIGINT || WTERMSIG(ret) == SIGQUIT)){
    	        return FALSE;
	     	}
       	}
      }
    }
  return TRUE;
  }

// Compress TAR
void ArchiveDialog::CompressTAR(fxmmFileList* panel){
#ifndef DISABLE_TAR
    dlg_cmd="tar -cf ";
	dlg_ext =".tar ";
	dlg_label="COMPRESSING: tar -cf";
	dlg_form=1;
	if(!executeCompressCommand(panel, dlg_cmd, dlg_ext, dlg_label, dlg_form)){
  	  	FXMessageBox::error(this,MBOX_OK,"COMPRESSION NOT EXECUTED","Operation Failed!");
	}
#endif
  return;
  }

// Compress GZ
void ArchiveDialog::CompressGZ(fxmmFileList* panel){
#ifndef DISABLE_GZIP
    dlg_cmd="gzip ";
	dlg_ext =".gz ";
	dlg_label="COMPRESSING: gzip";
	dlg_form=2;
	if(!executeCompressCommand(panel, dlg_cmd, dlg_ext, dlg_label, dlg_form)){
  	  	FXMessageBox::error(this,MBOX_OK,"COMPRESSION NOT EXECUTED","Operation Failed!");
	}
#endif
  return;
  }

// Compress BZ2
void ArchiveDialog::CompressBZ2(fxmmFileList* panel){
#ifndef DISABLE_BZIP2
    dlg_cmd="bzip2 -z ";
	dlg_ext =".bz2 ";
	dlg_label="COMPRESSING: bzip2 -z";
	dlg_form=2;
	if(!executeCompressCommand(panel, dlg_cmd, dlg_ext, dlg_label, dlg_form)){
  	  	FXMessageBox::error(this,MBOX_OK,"COMPRESSION NOT EXECUTED","Operation Failed!");
	}
#endif
  return;
  }

// Compress XZ
void ArchiveDialog::CompressXZ(fxmmFileList* panel){
#ifndef DISABLE_XZIP
    dlg_cmd="xz -z ";
	dlg_ext =".xz ";
	dlg_label="COMPRESSING: xz -z";
	dlg_form=2;
	if(!executeCompressCommand(panel, dlg_cmd, dlg_ext, dlg_label, dlg_form)){
  	  	FXMessageBox::error(this,MBOX_OK,"COMPRESSION NOT EXECUTED","Operation Failed!");
	}
#endif
  return;
  }

// Compress ZIP
void ArchiveDialog::CompressZIP(fxmmFileList* panel){
#ifndef DISABLE_ZIP
    dlg_cmd="zip ";
	dlg_ext =".zip -xi ";
	dlg_label="COMPRESSING: zip -xi";
	dlg_form=1;
	if(!executeCompressCommand(panel, dlg_cmd, dlg_ext, dlg_label, dlg_form)){
  	  	FXMessageBox::error(this,MBOX_OK,"COMPRESSION NOT EXECUTED","Operation Failed!");
	}
#endif
  return;
  }

// Compress 7Z
void ArchiveDialog::Compress7ZIP(fxmmFileList* panel){
#ifndef DISABLE_SEVENZIP
    dlg_cmd="7z a ";
	dlg_ext =".7z ";
	dlg_label="COMPRESSING: 7z a";
	dlg_form=1;
	if(!executeCompressCommand(panel, dlg_cmd, dlg_ext, dlg_label, dlg_form)){
  	  	FXMessageBox::error(this,MBOX_OK,"COMPRESSION NOT EXECUTED","Operation Failed!");
	}
#endif
  return;
  }


// Compress TAR
long ArchiveDialog::onCmdCompressTAR(FXObject*,FXSelector,void*){
    CompressTAR(filelist);
    return 1;
  }

// Compress XZ
long ArchiveDialog::onCmdCompressXZ(FXObject*,FXSelector,void*){
	CompressXZ(filelist);
  	return 1;
  }

// Compress ZIP
long ArchiveDialog::onCmdCompressZIP(FXObject*,FXSelector,void*){
	CompressZIP(filelist);
  	return 1;
  }

// Compress 7Z
long ArchiveDialog::onCmdCompress7ZIP(FXObject*,FXSelector,void*){
	Compress7ZIP(filelist);
    return 1;
  }

// Compress GZ
long ArchiveDialog::onCmdCompressGZ(FXObject*,FXSelector,void*){
	CompressGZ(filelist);
    return 1;
  }

// Compress BZ2
long ArchiveDialog::onCmdCompressBZ2(FXObject*,FXSelector,void*){
	CompressBZ2(filelist);
    return 1;
  }



// Extract archive
FXbool ArchiveDialog::extractArchive(FXString& file){
	FXString command,filename,rootname;

	// determine extension
   	filename=FXPath::absolute(filelist->getDirectory(),file);
	FXString ext=FXPath::extension(filename);
	printf("ext: %s\n", ext.text());

	// determine sub-extension
	rootname=FXPath::stripExtension(file);
	FXString root=FXPath::extension(rootname);
	printf("root: %s\n", root.text());


  	// Determine type of archive
	if(comparecase(ext,"gz")==0){
		if(comparecase(root,"tar")==0){
			command="tar xf "+file+" --gunzip &";
		}
		else if(comparecase(root,"tar")!=0){
			command="gunzip "+file+" &";
		}
	}
	else if(comparecase(ext,"bz2")==0){
		if(comparecase(root,"tar")==0){
			command="tar xf "+file+" --bzip2 &";
		}
		else if(comparecase(root,"tar")!=0){
			command="bunzip2 "+file+" &";
		}
	}
	else if(comparecase(ext,"xz")==0){
		if(comparecase(root,"tar")==0){
			command="tar xf "+file+" --xz &";
		}
		else if(comparecase(root,"tar")!=0){
			command="xz -d "+file+" &";
		}
	}
	else if(comparecase(ext,"lzma")==0){
		if(comparecase(root,"tar")==0){
			command="tar xf "+file+" --lzma &";
		}
		else if(comparecase(root,"tar")!=0){
			command="lzma -d "+file+" &";
		}
	}
	else if(comparecase(ext,"tgz")==0){
		command="tar -zxf "+file+" &";
	}
	else if(comparecase(ext,"tar")==0){
		command="tar -xf "+file+" &";
	}
	else if(comparecase(ext,"zip")==0){
		FXString pswd;
		if(FXInputDialog::getString(pswd,this,"ZIP PASSWORD","type password or press cancel")){
			if (comparecase (pswd, "")!=0){
				command="unzip "+file+" -P "+pswd+" &";
				//printf("command: %s\n", command.text());
			}
		}
		else{
			command="unzip "+file+" &";
		}
	}
	else if(comparecase(ext,"rar")==0){
		FXString pswd;
		if(FXInputDialog::getString(pswd,this,"RAR PASSWORD","type password or press cancel")){
			if (comparecase (pswd, "")!=0){
				command="unrar x "+file+" -p["+pswd+"] &";
				//printf("command: %s\n", command.text());
			}
		}
		else{
			command="unrar x "+file+" &";
		}
	}
	else if(comparecase(ext,"7z")==0){
		command="7z x "+file+" &";
	}
	else{
		FXMessageBox::error(this,MBOX_OK,"COMMAND FAILURE","Not suitable extension found %s\n",root.text());
		return FALSE;
	}

	FXString executable="cd "+FXPath::enquote(filelist->getDirectory())+"; "+command;

	FXDialogBox dialog(this,"EXTRACTING",DECOR_TITLE|DECOR_BORDER);
	FXVerticalFrame* content=new FXVerticalFrame(&dialog,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,10,10,10,10,10,10);
	new FXLabel(content,file,NULL,LAYOUT_FILL_X);
	FXHorizontalFrame* buttons=new FXHorizontalFrame(content,LAYOUT_FILL_X|PACK_UNIFORM_WIDTH,0,0,0,0,0,0,0,0);
	new FXButton(buttons,"&YES",accepticon,&dialog,FXDialogBox::ID_ACCEPT,BUTTON_INITIAL|ICON_BEFORE_TEXT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT);
	new FXButton(buttons,"&Cancel",cancelicon,&dialog,FXDialogBox::ID_CANCEL,ICON_BEFORE_TEXT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT,0,0,0,0,20,20);
	dialog.create();

	if(dialog.execute()){
   		int ret = system(executable.text());
    	if (WIFSIGNALED(ret) && (WTERMSIG(ret) == SIGINT || WTERMSIG(ret) == SIGQUIT)){
   	    	return FALSE;
    	}
	}
  return TRUE;
  }



long ArchiveDialog::onCmdExtract(FXObject*,FXSelector,void*){

  for(FXint i=0; i<filelist->getNumItems(); i++){
    if(filelist->isItemSelected(i)){
        FXString file=filelist->getItemFilename(i);
        if(file=="..") continue;

		// determine extension
    	filename=FXPath::absolute(filelist->getDirectory(),file);
        FXString ext=FXPath::extension(FXPath::name(filename));
        // printf("extension: %s\n",ext.text());
        if(comparecase(ext,"gz")==0||comparecase(ext,"xz")==0||comparecase(ext,"lzma")==0||comparecase(ext,"bz2")==0||comparecase(ext,"zip")==0||comparecase(ext,"tgz")==0||comparecase(ext,"7z")==0||comparecase(ext,"rar")==0||comparecase(ext,"tar")==0){
          extractArchive(filename);
        }
    }
  }
  return 1;
}


long ArchiveDialog::onCmdForceExtract(FXObject*,FXSelector,void*){
  FXString extrcommand=getApp()->reg().readStringEntry("FXArchive","extrcommand");
  for(FXint i=0; i<filelist->getNumItems(); i++){
    if(filelist->isItemSelected(i)){
      FXString file=filelist->getItemFilename(i);
      if(file=="..") continue;
      filename=FXPath::absolute(filelist->getDirectory(),file);

	  if(FXInputDialog::getString(extrcommand,this,"CUSTOM","extracting command:")){
		getApp()->reg().writeStringEntry("FXArchive","extrcommand",extrcommand.text());
	    extrcommand=extrcommand+ " "+file+" &";
		FXString executable="cd "+FXPath::enquote(filelist->getDirectory())+"; "+extrcommand;

		// do not determine extension but extract useing the given command
  		FXDialogBox dialog(this,"EXTRACTING",DECOR_TITLE|DECOR_BORDER);
		FXVerticalFrame* content=new FXVerticalFrame(&dialog,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,10,10,10,10,10,10);
  		new FXLabel(content,file,NULL,LAYOUT_FILL_X);
  		FXHorizontalFrame* buttons=new FXHorizontalFrame(content,LAYOUT_FILL_X|PACK_UNIFORM_WIDTH,0,0,0,0,0,0,0,0);
  		new FXButton(buttons,"&YES",accepticon,&dialog,FXDialogBox::ID_ACCEPT,BUTTON_INITIAL|ICON_BEFORE_TEXT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT);
  		new FXButton(buttons,"&Cancel",cancelicon,&dialog,FXDialogBox::ID_CANCEL,ICON_BEFORE_TEXT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT,0,0,0,0,20,20);
  		dialog.create();

  		if(dialog.execute()){
    	  int ret = system(executable.text());
          if (WIFSIGNALED(ret) && (WTERMSIG(ret) == SIGINT || WTERMSIG(ret) == SIGQUIT)){
    	    return FALSE;
	      }
    	}
	  }
    }
  }
  return 1;
}


// Popup menu for item in file list
long ArchiveDialog::onFileListPopup(FXObject*,FXSelector,void* ptr){
  FXEvent *event=(FXEvent*)ptr;
  if(event->moved) return 1;
  FXint index=filelist->getItemAt(event->win_x,event->win_y);
  FXMenuPane pane(filelist);
  FXMenuPane *compressmenu;

  // We clicked in the background
  if(index<0){
    FXLabel *filebacklabel = new FXLabel(&pane,"Background");
    filebacklabel->setTextColor(FXRGB(120,0,0));
    new FXMenuCommand(&pane," Refresh",m_refreshicon,filelist,FXFileList::ID_REFRESH);
    new FXMenuCommand(&pane," Terminal",m_terminalicon,this,ID_TERMINAL);
    new FXMenuCommand(&pane," Run here",m_consoleicon,this,ID_COMMAND);
    new FXMenuCommand(&pane," Paste",m_pasteicon,this,ID_CLIPBOARD_PASTE);
    }

  // We clicked on an item file
  else{
    FXLabel *fileforelabel = new FXLabel(&pane,"File Item");
    fileforelabel->setTextColor(FXRGB(120,0,0));

    new FXMenuCommand(&pane," Extract",extracticon,this,ID_EXTRACT);
    new FXMenuCommand(&pane," Command",commandicon,this,ID_FORCEEXTRACT);
    new FXMenuSeparator(&pane);

#ifndef DISABLE_TAR
    new FXMenuCommand(&pane," Tar",compressicons2,this,ID_COMPRESS_TAR);
#endif
    // Compress menu
    compressmenu=new FXMenuPane(&pane);
#ifndef DISABLE_GZIP
    new FXMenuCommand(compressmenu," GZIP",compressicons3,this,ID_COMPRESS_GZ);
#endif
#ifndef DISABLE_BZIP2
    new FXMenuCommand(compressmenu," BZIP2",compressicons4,this,ID_COMPRESS_BZ2);
#endif
#ifndef DISABLE_XZIP
    new FXMenuCommand(compressmenu," XZIP",compressicons1,this,ID_COMPRESS_XZ);
#endif
#ifndef DISABLE_SEVENZIP
    new FXMenuCommand(compressmenu," 7ZIP",compressicons5,this,ID_COMPRESS_7ZIP);
#endif
#ifndef DISABLE_ZIP
    new FXMenuCommand(compressmenu," ZIP",compressicons5,this,ID_COMPRESS_ZIP);
#endif
    new FXMenuCascade(&pane," Compress",compressicons1,compressmenu);

    new FXMenuSeparator(&pane);
    new FXMenuCommand(&pane," Open",m_appendicon,this,ID_OPEN);
    new FXMenuCommand(&pane," Open with",m_appendicon,this,ID_OPENWITH);
    new FXMenuSeparator(&pane);
    new FXMenuCommand(&pane," Rename",m_copyicon,this,ID_FILERENAME);
    new FXMenuCommand(&pane," Copy",m_copyicon,this,ID_CLIPBOARD_COPY);
    new FXMenuCommand(&pane," Cut",m_cuticon,this,ID_CLIPBOARD_CUT);
    new FXMenuSeparator(&pane);
    new FXMenuCommand(&pane," Delete",m_deleteicon,this,ID_FILEDELETE);
    }

  pane.create();
  pane.popup(NULL,event->root_x,event->root_y);
  getApp()->runModalWhileShown(&pane);
  return 1;
  }



// Pop up terminal
long ArchiveDialog::onCmdTerminal(FXObject*,FXSelector,void*){
  FXString executable="cd "+FXPath::enquote(filelist->getDirectory())+"; "+terminal+" &";
  system(executable.text());
  return 1;
  }



long ArchiveDialog::onCmdCommand(FXObject*,FXSelector,void*){
  FXString archcommand=getApp()->reg().readStringEntry("FXArchive","archcommand","md5sum fxmm &> fxmm.sum");

  if(FXInputDialog::getString(archcommand,this,"Run a command here","Command:")){
    getApp()->reg().writeStringEntry("FXArchive","archcommand",archcommand.text());
    archcommand="cd "+FXPath::enquote(filelist->getDirectory())+"; "+archcommand+" &";
    // printf("runcommand: %s\n", archcommand.text());
    system(archcommand.text());
    }
  return 1;
}


// Open with associated Command
long ArchiveDialog::onCmdOpen(FXObject*,FXSelector,void*){
  FXint index=filelist->getCurrentItem();
  if(index>=0){

    // If directory, open the directory
    if(filelist->isItemDirectory(index)){
      FXString directory=filelist->getItemPathname(index);
      FXTRACE((100,"directory=%s\n",directory.text()));
      filelist->setDirectory(directory);
      dirlist->setDirectory(directory);
      }

    // If regular file return as the selected file
    else if(filelist->isItemFile(index)){

      FXFileAssoc *fileassoc=filelist->getItemAssoc(index);
      if(fileassoc){
        if(fileassoc->command.text()){
          FXString command=FXString::value(fileassoc->command.text());
	      FXString runcommand="cd "+FXPath::enquote(filelist->getDirectory())+"; "+command+" "+filelist->getItemFilename(index)+" &";
          FXTRACE((100,"system(%s)\n",command.text()));
	      // printf("runcommand: %s\n", runcommand.text()); 
          system(runcommand.text());
          }
        else{
          FXMessageBox::information(this,MBOX_OK,"Unknown Command","No command has been defined for this file: %s",filelist->getItemFilename(index).text());
          }
        }
      else{
        FXMessageBox::information(this,MBOX_OK,"Unknown Mime Type","No Association has been set for this file: %s",filelist->getItemFilename(index).text());
        }
      }
  }
  return 1;
}



// Open with program
long ArchiveDialog::onCmdOpenWith(FXObject*,FXSelector,void*){
  FXString command=getApp()->reg().readStringEntry("SETTINGS","command","fxite");
  FXString filename=filelist->getCurrentFile();

  if(FXInputDialog::getString(command,this,"Open File","Open " + FXPath::name(filename) + " with:")){
    getApp()->reg().writeStringEntry("SETTINGS","command",command.text());
    command=command+" "+filename+" &";
    // printf("command: %s\n", command.text()); 
    system(command.text());
    }
  return 1;
}



// Close dialog with a cancel
long ArchiveDialog::onCmdQuit(FXObject*,FXSelector,void*){
  // Write new window size and position back to registry
#ifdef HAVE_LIBFOX_1_7
  getApp()->reg().writeUIntEntry("FXArchive","width",(FXuint)getWidth());
  getApp()->reg().writeUIntEntry("FXArchive","height",(FXuint)getHeight());
#endif
#ifdef HAVE_LIBFOX_1_6
  getApp()->reg().writeUnsignedEntry("FXArchive","width",(FXuint)getWidth());
  getApp()->reg().writeUnsignedEntry("FXArchive","height",(FXuint)getHeight());
#endif
  getApp()->reg().writeIntEntry("FXArchive","xpos",getX());
  getApp()->reg().writeIntEntry("FXArchive","ypos",getY());
  // set currentdir
  currentdir = dirlist->getDirectory();
  getApp()->reg().writeStringEntry("FXArchive","currentdir",currentdir.text());
  // Write registry settings
  getApp()->reg().write();
  // Quit
  getApp()->exit(0);
}



// Command when double clicking in the filelist
long ArchiveDialog::onCmdFilelist(FXObject*,FXSelector,void* ptr){
  FXint index=(FXint)(FXival)ptr;

  if(0<=index){
    if(filelist->isItemDirectory(index)){
      currentdir=filelist->getItemPathname(index);
      filelist->setDirectory(currentdir);
      dirlist->setDirectory(currentdir);
      }
    else if(filelist->isItemFile(index)){
      filename=filelist->getItemPathname(index);
	  FXString ext=FXPath::extension(FXPath::name(filename));
	  // printf("extension: %s\n",ext.text());
	  // autoextract if double clicking
	  if(comparecase(ext,"gz")==0||comparecase(ext,"xz")==0||comparecase(ext,"lzma")==0||comparecase(ext,"bz2")==0||comparecase(ext,"zip")==0||comparecase(ext,"tgz")==0||comparecase(ext,"7z")==0||comparecase(ext,"rar")==0||comparecase(ext,"tar")==0){
		extractArchive(filename);
	  }
    }
  }
  return 1;
}



// Change the directory in the filelist when clicking in the dirlist
long ArchiveDialog::onCmdDirlist(FXObject*,FXSelector,void* ptr){
  FXTreeItem *item=(FXTreeItem*)ptr;
  FXString path=dirlist->getItemPathname(item);
  filelist->setDirectory(path);
  return 1;
  }


// Enable sender when Files have been selected
long ArchiveDialog::onUpdSelectedCompress(FXObject* sender,FXSelector,void*){
    FXString filename, ext;
    for(FXint i=0; i<filelist->getNumItems(); i++){
		if(filelist->isItemSelected(i)){
			if(filelist->isItemDirectory(i)){
				sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_DISABLE),NULL);
				return 1;
			}else{
       			filename=filelist->getItemFilename(i);
	  			ext=FXPath::extension(FXPath::name(filename));
	  			// printf("extension: %s\n",ext.text());
	  			if(comparecase(ext,"gz")==0||comparecase(ext,"xz")==0||comparecase(ext,"lzma")==0||comparecase(ext,"bz2")==0||comparecase(ext,"zip")==0||comparecase(ext,"tgz")==0||comparecase(ext,"7z")==0||comparecase(ext,"rar")==0){
					sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_DISABLE),NULL);
					return 1;
				}
			}
		}
	}
    sender->handle(this,getNumFilenames(filelist)?FXSEL(SEL_COMMAND,ID_ENABLE):FXSEL(SEL_COMMAND,ID_DISABLE),NULL);
    return 1;
}


// Enable sender when Files have been selected
long ArchiveDialog::onUpdSelectedTar(FXObject* sender,FXSelector,void*){
    FXString filename, ext;
    for(FXint i=0; i<filelist->getNumItems(); i++){
		if(filelist->isItemSelected(i)){
   			filename=filelist->getItemFilename(i);
  			ext=FXPath::extension(FXPath::name(filename));
  			// printf("extension: %s\n",ext.text());
	  		if(comparecase(ext,"gz")==0||comparecase(ext,"xz")==0||comparecase(ext,"lzma")==0||comparecase(ext,"bz2")==0||comparecase(ext,"zip")==0||comparecase(ext,"tgz")==0||comparecase(ext,"7z")==0||comparecase(ext,"rar")==0||comparecase(ext,"tar")==0){
				sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_DISABLE),NULL);
				return 1;
			}
		}
	}
    sender->handle(this,getNumFilenames(filelist)?FXSEL(SEL_COMMAND,ID_ENABLE):FXSEL(SEL_COMMAND,ID_DISABLE),NULL);
    return 1;
}


// Enable sender when Files have been selected
long ArchiveDialog::onUpdSelectedExtract(FXObject* sender,FXSelector,void*){
	FXString filename, ext;
    for(FXint i=0; i<filelist->getNumItems(); i++){
		if(filelist->isItemSelected(i)){
			filename=filelist->getItemFilename(i);
	  		ext=FXPath::extension(FXPath::name(filename));
	  		//printf("extension: %s\n",ext.text());
	  		if(comparecase(ext,"gz")!=0&&comparecase(ext,"xz")!=0&&comparecase(ext,"lzma")!=0&&comparecase(ext,"bz2")!=0&&comparecase(ext,"zip")!=0&&comparecase(ext,"tgz")!=0&&comparecase(ext,"7z")!=0&&comparecase(ext,"tar")!=0&&comparecase(ext,"rar")!=0){
				sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_DISABLE),NULL);
				return 1;
			}
		}
	}
    sender->handle(this,getNumFilenames(filelist)?FXSEL(SEL_COMMAND,ID_ENABLE):FXSEL(SEL_COMMAND,ID_DISABLE),NULL);
    return 1;
}


// rename
long ArchiveDialog::onCmdFileRename(FXObject*,FXSelector,void*){
	FXbool overwrite_file=TRUE;
	FXString filename=filelist->getCurrentFile();
  	if(FXStat::exists(filename)){
		FXDialogBox dialog(this,"Rename File",DECOR_TITLE|DECOR_BORDER);
  		FXVerticalFrame* content=new FXVerticalFrame(&dialog,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,10,10,10,10,10,10);
  		new FXLabel(content,"Old Name: "+FXPath::absolute(FXPath::directory(filename),FXPath::name(filename)),NULL,LAYOUT_FILL_X|JUSTIFY_LEFT);
		FXMatrix *matrix = new FXMatrix(content,2,MATRIX_BY_COLUMNS|LAYOUT_FILL_X|LAYOUT_FILL_Y);
		new FXLabel(matrix,"Root:");
  		FXTextField *trootname=new FXTextField(matrix,40,&dialog,FXDialogBox::ID_ACCEPT,TEXTFIELD_ENTER_ONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X);
		new FXLabel(matrix,"Name:");
  		FXTextField *tnewname=new FXTextField(matrix,40,&dialog,FXDialogBox::ID_ACCEPT,TEXTFIELD_ENTER_ONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X);
  		new FXHorizontalSeparator(content,SEPARATOR_GROOVE|LAYOUT_FILL_X);
  		FXHorizontalFrame* buttons=new FXHorizontalFrame(content,LAYOUT_FILL_X|PACK_UNIFORM_WIDTH,0,0,0,0,0,0,0,0);
  		new FXButton(buttons,"&Yes",accepticon,&dialog,FXDialogBox::ID_ACCEPT,BUTTON_INITIAL|ICON_BEFORE_TEXT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT);
  		new FXButton(buttons,"&No",cancelicon,&dialog,FXDialogBox::ID_CANCEL,ICON_BEFORE_TEXT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT,0,0,0,0,20,20);
  		FXString destinationdir=filelist->getDirectory();
  		trootname->setText(FXPath::absolute(destinationdir)+"/");
  		tnewname->setText(FXPath::name(filename));
  		tnewname->setFocus();

  		if(dialog.execute()){
      			FXString rootname=trootname->getText();
      			FXString newname=tnewname->getText();
    			if(FXStat::exists(rootname+newname)){
      	  			if(MBOX_CLICKED_NO==FXMessageBox::question(this,MBOX_YES_NO,"Already Exists","Overwrite Existing: %s?",filename.text())) overwrite_file=FALSE;
    			}
			if(!FXFile::moveFiles(filename,FXPath::absolute(FXPath::directory(filename),rootname+newname),overwrite_file)){
	   			if(FXMessageBox::error(this,MBOX_OK,"Error Renaming","Unable to Rename:\n\n%s  to:  %s\n\n",filename.text(),(rootname+newname).text())==MBOX_CLICKED_OK) return 0;
			}else{
				FXFile::moveFiles(filename,FXPath::absolute(FXPath::directory(filename),rootname+newname),overwrite_file);
			}
   		}
	}
  return 1;
}


// delete
long ArchiveDialog::onCmdFileDelete(FXObject*,FXSelector,void*){
  FXint count=0;
  FXString logfiles,filetoremove,file;

  for(FXint i=0; i<filelist->getNumItems(); i++){
    if(filelist->isItemSelected(i)){
        file=filelist->getItemFilename(i);
        if(file=="..") continue;
	if(count>20) {logfiles+=".........................\n\nDelete more than 20 files?\n"; break;}
	logfiles+=file+"\n";
        count=count+1;
    }
  }

  FXuint answer=FXMessageBox::warning(this,MBOX_YES_NO,"Deleting Movie Files",logfiles.text());
  if(answer==MBOX_CLICKED_YES){

    for(FXint i=0; i<filelist->getNumItems(); i++){
      if(filelist->isItemSelected(i)){
        file=filelist->getItemFilename(i);
        if(file=="..") continue;
        filetoremove=FXPath::absolute(filelist->getDirectory(),file);
        FXTRACE((100,"filetoremove=%s\n",filetoremove.text()));
        if(!FXFile::removeFiles(filetoremove,TRUE)){
          if(MBOX_CLICKED_NO==FXMessageBox::error(this,MBOX_YES_NO,"Error Deleting Movie","Unable to delete movie: %s\nContinue with operation?",filetoremove.text())){
            break;
            }
          }
        }
      }
    }
  return 1;
  }


// Enable sender when Items have been selected
long ArchiveDialog::onUpdFileSelected(FXObject* sender,FXSelector,void*){
    sender->handle(this,getNumFilenames(filelist)?FXSEL(SEL_COMMAND,ID_ENABLE):FXSEL(SEL_COMMAND,ID_DISABLE),NULL);
    return 1;
  }



// Get number of selected filenames, not including "." and ".."
FXint ArchiveDialog::getNumFilenames(fxmmFileList* filelist) const {
  register FXint num=0;
  register FXint i;
  if(filelist->getNumItems()){
    for(i=0; i<filelist->getNumItems(); i++){
#ifdef HAVE_LIBFOX_1_6
      if(filelist->isItemSelected(i) && filelist->getItemFilename(i)!=".." && filelist->getItemFilename(i)!=".") num++;
#endif
#ifdef HAVE_LIBFOX_1_7
      if(filelist->isItemSelected(i) && !filelist->isItemNavigational(i)) num++;
#endif
      }
    }
  return num;
  }


// Update title with current directory viewed
long ArchiveDialog::onUpdTitle(FXObject* sender,FXSelector,void*){

  FXString title="FXArchive View: " + filelist->getDirectory();
  sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_SETSTRINGVALUE),(void*)&title);
  return 1;
  }


// Switch to given directory
void ArchiveDialog::setDirectory(const FXString& dirname){
  FXString newpath=FXPath::absolute(dirname);
  filelist->setDirectory(newpath);
  dirlist->setDirectory(newpath);
  }



// create application
void ArchiveDialog::create() {
    // Make file associations object;
    // shared between FXFileList and FXDirList
	if(!currentdir.empty()){
      dirlist->setDirectory(currentdir);
      filelist->setDirectory(currentdir);
    }

    if(!fontspec.empty()){
      bodyfont=new FXFont(getApp(),fontspec);
      filelist->setFont(bodyfont);
      dirlist->setFont(bodyfont);
    }

    if(!fontspec2.empty()){
      menufont=new FXFont(getApp(),fontspec2);
      getApp()->setNormalFont(menufont);
      button_ta->setFont(menufont);
      button_gz->setFont(menufont);
      button_bz->setFont(menufont);
      button_zi->setFont(menufont);
      button_7z->setFont(menufont);
      button_xz->setFont(menufont);
      button_ex->setFont(menufont);
      button_cm->setFont(menufont);
      button_qu->setFont(menufont);
      title_arrange->setFont(menufont);
      radio_det->setFont(menufont);
      radio_sma->setFont(menufont);
      radio_big->setFont(menufont);
      title_show->setFont(menufont);
	  check_fil->setFont(menufont);
	  check_dir->setFont(menufont);
	  check_img->setFont(menufont);
      title_about->setFont(menufont);
	  cmd_about->setFont(menufont);
    }

  	getApp()->setForeColor(theme_fxmmfore);
  	getApp()->setBaseColor(theme_fxmmbase);
  	getApp()->setBackColor(theme_fxmmback);
  	getApp()->setBorderColor(theme_fxmmborder);
  	getApp()->setSelMenuTextColor(theme_fxmmmenufore);
  	getApp()->setSelMenuBackColor(theme_fxmmmenuback);
  	getApp()->setSelforeColor(theme_fxmmselfore);
  	getApp()->setSelbackColor(theme_fxmmselback);
  	getApp()->setTipforeColor(theme_fxmmtipfore);
  	getApp()->setTipbackColor(theme_fxmmtipback);
	getApp()->setHiliteColor(theme_fxmmhilite);
	getApp()->setShadowColor(theme_fxmmshadow);
	
	
    filelist->setHeaderIcon(0,m_renamediricon);
    filelist->setHeaderIcon(1,m_diricon);
    filelist->setHeaderIcon(2,m_dimicon);
    filelist->setHeaderIcon(3,m_daticon);
    filelist->setHeaderIcon(4,m_usricon);
    filelist->setHeaderIcon(5,m_grpicon);
    filelist->setHeaderIcon(6,m_prmicon);
    filelist->setHeaderIcon(7,m_consoleicon);

    filelist->setHeaderText(0,file_n);
    filelist->setHeaderText(1,file_t);
    filelist->setHeaderText(2,file_s);
    filelist->setHeaderText(3,file_m);
    filelist->setHeaderText(4,file_u);
    filelist->setHeaderText(5,file_g);
    filelist->setHeaderText(6,file_a);
    filelist->setHeaderText(7,file_l);

    associations=new FXFileDict(getApp());
    filelist->setAssociations(associations);
    dirlist->setAssociations(associations);

    // Register standard uri-list type
	urilistType=getApp()->registerDragType("text/uri-list");
	actionType=getApp()->registerDragType("text/clip-action");

    FXMainWindow::create();
    show(PLACEMENT_SCREEN);

    return;
}



FXbool ArchiveDialog::readConfig() {
    // Read registry
	if(!getApp()->reg().read()){
		return false;
	}

    // Get size and position
#ifdef HAVE_LIBFOX_1_7
    FXuint width=getApp()->reg().readUIntEntry("FXArchive","width",700);
    FXuint height=getApp()->reg().readUIntEntry("FXArchive","height",400);
#endif
#ifdef HAVE_LIBFOX_1_6
    FXuint width=getApp()->reg().readUnsignedEntry("FXArchive","width",700);
    FXuint height=getApp()->reg().readUnsignedEntry("FXArchive","height",400);
#endif

    FXint xpos=getApp()->reg().readIntEntry("FXArchive","xpos",40);
    FXint ypos=getApp()->reg().readIntEntry("FXArchive","ypos",5);

    // Reposition window
    position(xpos,ypos,width,height);

    currentdir=getApp()->reg().readStringEntry("FXArchive","currentdir");

	// Read Colors
  	if(! (theme_fxmmfore=getApp()->reg().readColorEntry("SETTINGS","fxmmforecolor"))) { theme_fxmmfore   = FXRGB(180,180,180);}
  	if(! (theme_fxmmback=getApp()->reg().readColorEntry("SETTINGS","fxmmbackcolor"))) { theme_fxmmback   = FXRGB( 20, 20, 20);}
  	if(! (theme_fxmmbase=getApp()->reg().readColorEntry("SETTINGS","fxmmbasecolor"))) { theme_fxmmbase   = FXRGB( 60, 60, 60);}
  	if(! (theme_fxmmborder=getApp()->reg().readColorEntry("SETTINGS","fxmmbordercolor"))) { theme_fxmmborder = FXRGB(  0,  0,  0);}
  	if(! (theme_fxmmmenufore=getApp()->reg().readColorEntry("SETTINGS","fxmmmenuforecolor"))) { theme_fxmmmenufore  = FXRGB(240,240,240);}
  	if(! (theme_fxmmmenuback=getApp()->reg().readColorEntry("SETTINGS","fxmmmenubackcolor"))) { theme_fxmmmenuback  = FXRGB( 10, 10, 10);}
  	if(! (theme_fxmmselfore=getApp()->reg().readColorEntry("SETTINGS","fxmmselforecolor"))) { theme_fxmmselfore   = FXRGB(  0,  0,  0);}
  	if(! (theme_fxmmselback=getApp()->reg().readColorEntry("SETTINGS","fxmmselbackcolor"))) { theme_fxmmselback   = FXRGB(140,140,140);}
  	if(! (theme_fxmmtipfore=getApp()->reg().readColorEntry("SETTINGS","fxmmtipforecolor"))) { theme_fxmmtipfore  = FXRGB(220,220,220);}
  	if(! (theme_fxmmtipback=getApp()->reg().readColorEntry("SETTINGS","fxmmtipbackcolor"))) { theme_fxmmtipback  = FXRGB( 20, 20, 20);}
  	if(! (theme_fxmmhilite=getApp()->reg().readColorEntry("SETTINGS","fxmmhilitecolor"))) { theme_fxmmhilite  = FXRGB(100,100,100);}
  	if(! (theme_fxmmshadow=getApp()->reg().readColorEntry("SETTINGS","fxmmshadowcolor"))) { theme_fxmmshadow  = FXRGB( 10, 10, 10);}


  language=getApp()->reg().readStringEntry("SETTINGS","language");
  if(language== "en" || language== "it" || language== "fr" || language== "de" || language== "es" || language== "ru" || language== "cn" || language== "jp" ){
  // Read language
    file_n=getApp()->reg().readStringEntry("LANG_MENU","file_n");
    file_t=getApp()->reg().readStringEntry("LANG_MENU","file_t");
    file_s=getApp()->reg().readStringEntry("LANG_MENU","file_s");
    file_m=getApp()->reg().readStringEntry("LANG_MENU","file_m");
    file_u=getApp()->reg().readStringEntry("LANG_MENU","file_u");
    file_g=getApp()->reg().readStringEntry("LANG_MENU","file_g");
    file_a=getApp()->reg().readStringEntry("LANG_MENU","file_a");
    file_l=getApp()->reg().readStringEntry("LANG_MENU","file_l");

  } else if(language=="cu"){
  // Read custom language
    file_n=getApp()->reg().readStringEntry("LANG_MENU_CUSTOM","file_n");
    file_t=getApp()->reg().readStringEntry("LANG_MENU_CUSTOM","file_t");
    file_s=getApp()->reg().readStringEntry("LANG_MENU_CUSTOM","file_s");
    file_m=getApp()->reg().readStringEntry("LANG_MENU_CUSTOM","file_m");
    file_u=getApp()->reg().readStringEntry("LANG_MENU_CUSTOM","file_u");
    file_g=getApp()->reg().readStringEntry("LANG_MENU_CUSTOM","file_g");
    file_a=getApp()->reg().readStringEntry("LANG_MENU_CUSTOM","file_a");
    file_l=getApp()->reg().readStringEntry("LANG_MENU_CUSTOM","file_l");
  }
    // Read fonts
    fontspec=getApp()->reg().readStringEntry("SETTINGS","bodyfont","");
    fontspec2=getApp()->reg().readStringEntry("SETTINGS","menufont","");

	// Read terminal
    terminal=getApp()->reg().readStringEntry("SETTINGS","terminal");

    // Read registry settings
	if(getApp()->reg().read()){
		return true;
	}
}


/***************** BASIC CLIPBOARD MANAGEMENT *****************/

// The selection has been lost (Paste after Copy/Cut or Copy/Cut after Copy/Cut)
long ArchiveDialog::onClipboardLost(FXObject* sender,FXSelector sel,void* ptr) {
	// BaseClassWidget received SEL_CLIPBOARD_LOST msg
	FXMainWindow::onClipboardLost(sender,sel,ptr);
	printf("Clipboard: Lost: %s\n", clipboard.text());
	return 1;
}


// The selection has been gained
long ArchiveDialog::onClipboardGained(FXObject* sender,FXSelector sel,void* ptr) {
	// BaseClassWidget received SEL_CLIPBOARD_GAINED msg
	FXMainWindow::onClipboardGained(sender,sel,ptr);
	printf("Clipboard: Gained: %s\n", clipboard.text());
	return 1;
}


// The selection has been request 
long ArchiveDialog::onClipboardRequest(FXObject* sender,FXSelector sel,void* ptr) {
    // See if the base class knows how to deal with the requested clipboard uri list type
    if(FXMainWindow::onClipboardRequest(sender,sel,ptr)){
		return 1;
	}
	// See if we can deal with this uri list type ourselves
	if (((FXEvent*)ptr)->target==urilistType) {
		// Return the clipboard content
		if(!clipboard.empty()){
			setDNDData(FROM_CLIPBOARD,urilistType,clipboard);
		}
		// Return 1 because it was handled herere
		return 1;
	}
	// Return 0 to signify we haven't dealt with it yet;
	// a derived class of FXMainWindow may yet give another try....
	return 0;
}


// COPY or CUT to the clipboard 
long ArchiveDialog::onCmdCopyCut(FXObject*,FXSelector sel,void*) {		

	clipboard.clear();

	// Define the DragType: urilistType and actionType for every CUT/COPY cmd
	FXDragType dragtp[2]={urilistType,actionType};

	// Acquires the clipboard and send msg SEL_CLIPBOARD_LOST to the previous widget which owned the clipboard
	if (acquireClipboard(dragtp,2)){
    	// set the action type
		if (FXSELID(sel)==ID_CLIPBOARD_CUT){
			actionType=1;
		}else{
			actionType=0;
		}
		// If we have some files selected, go ahead
    	FXint n=getNumFilenames(filelist);
    	if (n==0){ 	return 0; }  // error we don't have any file selected
		else if (n>=1){
		// Construct the uri LIST of files and fill the clipboard with it
			for(int i=0; i<filelist->getNumItems(); i++){
				if(filelist->isItemSelected(i) && filelist->getItemFilename(i)!=".." && filelist->getItemFilename(i)!="."){
				clipboard.append(FXURL::fileToURL(filelist->getItemPathname(i)));
				}
			}
		}
		//printf("copyfiles: %s\n", clipboard.text());
	}
	return 1;
}


// PASTE from the clipboard
long ArchiveDialog::onCmdPaste(FXObject*,FXSelector sel,void*) {
    FXint n;
    FXString string, s_dst, s_beg, s_end;	  
	//printf("actionType: %i\n", actionType);

	// Check if the source has a drag type urilistType and get DND data; 
	if(getDNDData(FROM_CLIPBOARD,urilistType,string)){
		//printf("string: %s\n", string.text());
		n=string.contains("file://")+1;
		string.substitute("file://", "\n", true);
		//printf("string: %s\n", string.text());

		for(FXint i=2; i<n+1; i++){
			s_beg = string.before('\n',i);
			s_end = s_beg.after('\n',i-1);
			s_dst = filelist->getDirectory()+"/"+FXPath::name(s_end);
			//printf("endfiles: %s\n", s_end.text());
			//printf("destfiles: %s\n", s_dst.text());

			// *********************************************************************
			// actionType needs to be initialized for all apps sharing the clipboard
			// *********************************************************************
			if (!FXMainWindow::hasClipboard()){
				if(actionType!=1 && actionType!=0){
					actionType=0;
				}
			}
			if(actionType==1){
				if(!FXFile::moveFiles(s_end,s_dst,false)){
					if(FXMessageBox::question(this,MBOX_YES_NO,"Overwrite File","Overwrite existing file or directory: %s?",s_dst.text())==MBOX_CLICKED_NO){
						break;
					}
					FXFile::moveFiles(s_end,s_dst,true);
				}
			}else if(actionType==0){
				if(!FXFile::copyFiles(s_end,s_dst,false)){
					if(FXMessageBox::question(this,MBOX_YES_NO,"Overwrite File","Overwrite existing file or directory: %s?",s_dst.text())==MBOX_CLICKED_NO){
							break;
					}
					FXFile::copyFiles(s_end,s_dst,true);
				}
			}
		}
		// Clear the clipboard so that onUpdPaste() function gray out the Paste button
		clipboard.clear();
	}
	return 1;
}


// Update the PASTE button
long ArchiveDialog::onUpdPaste(FXObject* sender,FXSelector sel,void*) {
    FXbool clipempty=TRUE;
	FXString string;

	// Check if the source has a drag type urilistType and get DND data; 
	if(getDNDData(FROM_CLIPBOARD,urilistType,string)){
		// Check if clipboard has a valid string content
		if (string.contains("file://")>=0){
			clipempty=FALSE;
		}
	}
	// Now we know if the clipboard is empty: if so gray out the paste button
	if (clipempty){
		sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_DISABLE),NULL);
	}else{
		sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_ENABLE),NULL);
	}
    return 1;
}



// About box
long ArchiveDialog::onCmdAbout(FXObject*,FXSelector,void*){
  FXMessageBox about(this,"About FXArchive",
	"FXArchive is a Compression / Decompression Utility,\n"
	"Copyright (C) 2010,2012 Luigi Ziviani (ziviani_luigi@yahoo.it).\n\n"
	"This program is free software; you can redistribute it\n"
	"and/or modify it under the terms of the GNU General Public\n"
	"License as published by the Free Software Foundation;\n"
	"either version 3 of the License, or(at your option)\n"
	"any later version.\n\n"
	"This program is distributed in the hope that it will be useful,\n"
	"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
	"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n"
	"See the GNU General Public License for more details.\n\n"
	"You should have received a copy of the GNU General Public License\n"
	"along with this program; if not, write to the \n"
	"Free Software Foundation, Inc.,\n"
	"59 Temple Place, Suite 330, Boston, MA  02111-1307  USA",
	NULL,MBOX_OK|DECOR_TITLE|DECOR_BORDER);
	about.execute();
  return 1;
  }

// main
int main(int argc, char *argv[]) {
    // Make application
    FXApp application("Configuration","FXMM");
    // Open display
    application.init(argc,argv);
    // Make window: the new FXMovieManager(&application);
    ArchiveDialog* fxarchive = new ArchiveDialog(&application);
    // Handle interrupt to save stuff nicely
    application.addSignal(SIGINT,fxarchive,ArchiveDialog::ID_QUIT);
    // Create it
    application.create();
    // Start in given directory
    if(argc==2) fxarchive->setDirectory(argv[1]);
    // Run
    return application.run();
}


// EOF
